﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows;

namespace HierarchicalHeaderAlignment
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {

            InitializeComponent();
           
            Data = new ObservableCollection<Level0>();

            for (int i = 0; i < 10; i++)
            {
                Level0 lvl = new Level0
                {
                    Id = i,
                    Name = "Level0 - " + i,
                    Date = DateTime.Now.AddDays(i),
                    Type = i % 2 == 0 ? "Type " + i : "",
                    Description = i % 2 != 0 ? "Description " + i : "",
                    ChildLevel1 = new ObservableCollection<Level1>()
                };

                for (int j = 0; j < 5; j++)
                {
                    Level1 lvl1 = new Level1
                    {
                        Name = "Lavel1 - " + j,
                        Date = DateTime.Now.AddYears(j),
                        Type = "",
                        Description = "Level1 Description",
                        ChildLevel1 = new ObservableCollection<Level2>()
                    };

                    for (int p = 0; p < 5; p++)
                    {
                        Level2 lvl2 = new Level2
                        {
                            Name = "Level2 - " + p,
                            Date = DateTime.Now.AddYears(p),
                            Type = "Some Type" + p,
                            ChildLevel1 = new ObservableCollection<Level3>()
                        };

                        for (int q = 0; q < 5; q++)
                        {
                            lvl2.ChildLevel1.Add(new Level3 { Name = "Level3 - " + q, Date = DateTime.Now.AddDays(q), Type = "", Description = "Text" });
                        }

                        lvl1.ChildLevel1.Add(lvl2);
                    }

                    lvl.ChildLevel1.Add(lvl1);
                }

                Data.Add(lvl);
            }

            DataContext = this;
        }

        void Label_Loaded(object sender, RoutedEventArgs e)
        {
            LabelPresenter lp = sender as LabelPresenter;
            if (lp.Field.Index == 0)
            {
                double width = lp.Field.CellWidthResolved;
                int index = FieldLayoutIndexFromField(lp.Field, xamDataGrid1);

                if (index > -1 && index < 2 && lp.Field.Width == null)
                {
                    double final = width + 17 * (2 - index);
                    lp.Field.Width = new FieldLength(final);
                }
            }
        }

        void Label_SizeChanged(object sender, RoutedEventArgs e)
        {
            LabelPresenter lp = sender as LabelPresenter;
            int layoutIndex = FieldLayoutIndexFromField(lp.Field, xamDataGrid1);

            if (lp.Field.Index == 0)
            {
                if (layoutIndex == 0)
                {
                    for (int i = 0; i < xamDataGrid1.FieldLayouts.Count; i++)
                    {
                        if (i != layoutIndex)
                        {
                            if (i == 1)
                            {
                                Field f = this.xamDataGrid1.FieldLayouts[i].Fields[lp.Field.Index];
                                f.Width = new FieldLength(lp.Field.CellWidthResolved - 17);
                            }
                            else
                            {
                                Field f = this.xamDataGrid1.FieldLayouts[i].Fields[lp.Field.Index];
                                f.Width = new FieldLength(lp.Field.CellWidthResolved - 34);
                            }
                        }
                    }
                } 
                else if (layoutIndex == 1)
                {
                    for (int i = 0; i < xamDataGrid1.FieldLayouts.Count; i++)
                    {
                        if (i != layoutIndex)
                        {
                            if (i == 0)
                            {
                                Field f = this.xamDataGrid1.FieldLayouts[i].Fields[lp.Field.Index];
                                f.Width = new FieldLength(lp.Field.CellWidthResolved + 17);
                            }
                            else
                            {
                                Field f = this.xamDataGrid1.FieldLayouts[i].Fields[lp.Field.Index];
                                f.Width = new FieldLength(lp.Field.CellWidthResolved - 17);
                            }
                        }
                    }
                }
                else
                {
                    for (int i = 0; i < xamDataGrid1.FieldLayouts.Count; i++)
                    {
                        if (i != layoutIndex)
                        {
                            if (i < 2)
                            {
                                Field f = this.xamDataGrid1.FieldLayouts[i].Fields[lp.Field.Index];
                                f.Width = new FieldLength(lp.Field.CellWidthResolved + 17 * (2 - i));
                            }
                            else
                            {
                                Field f = this.xamDataGrid1.FieldLayouts[i].Fields[lp.Field.Index];
                                f.Width = new FieldLength(lp.Field.CellWidthResolved);
                            }
                        }
                    }
                }
            }
            else
            {
                for (int i = 0; i < xamDataGrid1.FieldLayouts.Count; i++)
                {
                    if (i != layoutIndex)
                    {
                        Field f = this.xamDataGrid1.FieldLayouts[i].Fields[lp.Field.Index];
                        f.Width = new FieldLength(lp.Field.CellWidthResolved);
                    }
                }
            }
        }

        void Record_Loaded(object sender, RoutedEventArgs e)
        {
            if ((sender as DataRecordPresenter).IsHeaderRecord)
            {
                Grid g = Utilities.GetDescendantFromType(sender as DependencyObject, typeof(Grid), false) as Grid;
                g.Height = 0;
            }
        }

        public ObservableCollection<Level0> Data { get; set; }

        public int FieldLayoutIndexFromField(Field f, XamDataGrid grid)
        {
            int index = -1;

            for (int i = 0; i < grid.FieldLayouts.Count; i++)
            {
                if (xamDataGrid1.FieldLayouts[i].Fields.Contains(f))
                {
                    index = i;
                    break;
                }
            }

            return index;
        }
        
    }

    public class Level0
    {
        public int Id { get; set; }
        public string Name { get; set; }
        public DateTime Date { get; set; }
        public string Description { get; set; }
        public string Type { get; set; }
        public ObservableCollection<Level1> ChildLevel1 { get; set; }
    }

    public class Level1
    {
        public int ID { get; set; }
        public string Name { get; set; }
        public DateTime Date { get; set; }
        public string Description { get; set; }
        public string Type { get; set; }
        public ObservableCollection<Level2> ChildLevel1 { get; set; }
    }

    public class Level2
    {
        public int ID { get; set; }
        public string Name { get; set; }
        public DateTime Date { get; set; }
        public string Description { get; set; }
        public string Type { get; set; }
        public ObservableCollection<Level3> ChildLevel1 { get; set; }
    }

    public class Level3
    {
        public int ID { get; set; }
        public string Name { get; set; }
        public DateTime Date { get; set; }
        public string Description { get; set; }
        public string Type { get; set; }
    }
}
